<?php

namespace Drupal\eca_report\Controller;

use Drupal\Component\Plugin\PluginInspectionInterface;
use Drupal\Component\Render\MarkupInterface;
use Drupal\Component\Serialization\Yaml;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Extension\ModuleExtensionList;
use Drupal\Core\Plugin\DefaultPluginManager;
use Drupal\Core\Plugin\PluginBase;
use Drupal\Core\Render\Markup;
use Drupal\Core\Url;
use Drupal\eca\Plugin\ECA\EcaPluginBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Base controller for ECA plugins report.
 */
abstract class EcaReportBaseController extends ControllerBase implements ContainerInjectionInterface {

  /**
   * Module extension list.
   */
  protected ModuleExtensionList $moduleExtensionList;

  /**
   * The ECA plugin manager (event, condition, or action).
   */
  protected DefaultPluginManager $pluginManager;

  /**
   * The ECA plugin type (event, condition, or action).
   */
  protected string $pluginType;

  /**
   * The documentation domain. May be NULL if not enabled or specified.
   */
  protected ?string $documentationDomain;

  /**
   * The documentation icon.
   */
  protected MarkupInterface $icon;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->moduleExtensionList = $container->get('extension.list.module');
    $instance->pluginManager = $container->get('plugin.manager.eca.' . $instance->pluginType);

    $instance->documentationDomain = $container->getParameter('eca.default_documentation_domain')
      ? $instance->config('eca.settings')->get('documentation_domain')
      : NULL;

    $instance->icon = Markup::create('<svg width="12" height="12" viewBox="0 0 12 12" fill="none" xmlns="http://www.w3.org/2000/svg">
      <path fill-rule="evenodd" clip-rule="evenodd" d="M10.6368 10.6375V5.91761H11.9995V10.6382C11.9995 10.9973 11.8623 11.3141 11.5878 11.5885C11.3134 11.863 10.9966 12.0002 10.6375 12.0002H1.36266C0.982345 12.0002 0.660159 11.8681 0.396102 11.6041C0.132044 11.34 1.52588e-05 11.0178 1.52588e-05 10.6375V1.36267C1.52588e-05 0.98236 0.132044 0.660173 0.396102 0.396116C0.660159 0.132058 0.982345 2.95639e-05 1.36266 2.95639e-05H5.91624V1.36267H1.36266V10.6375H10.6368ZM12 0H7.2794L7.27873 1.36197H9.68701L3.06507 7.98391L4.01541 8.93425L10.6373 2.31231V4.72059H12V0Z" fill="#818798"/>
    </svg>');

    return $instance;
  }

  /**
   * Displays an ECA plugin report.
   *
   * @return array
   *   A render array containing an ECA plugin report.
   */
  abstract public function index(): array;

  /**
   * Retrieves the usage of ECA plugins in ECA workflows.
   *
   * @return array
   *   An associative array where the keys are plugin IDs and the values are
   *   arrays of ECA workflows.
   */
  protected function getUsedIn(): array {
    /** @var \Drupal\eca\Entity\Eca[] $entities */
    $entities = \Drupal::entityTypeManager()
      ->getStorage('eca')
      ->loadMultiple();
    $used_in = [];
    foreach ($entities as $entity) {
      foreach ($entity->get($this->pluginType . 's') as $item) {
        $plugin_id = $item['plugin'];

        $used_in += [$plugin_id => []];
        $used_in[$plugin_id][$entity->id()] = [
          '#type' => 'link',
          '#title' => $entity->label(),
          '#url' => $entity->toUrl(),
        ];
      }
    }
    return $used_in;
  }

  /**
   * Builds a label representation for an ECA plugin.
   *
   * @param \Drupal\Core\Plugin\PluginBase|EcaPluginBase $plugin
   *   The ECA plugin instance.
   *
   * @return string|array
   *   A rendered array containing the label details if documentation URL exists,
   *   or a simple string label if the URL is not available.
   */
  protected function buildLabelDescription(PluginBase|EcaPluginBase $plugin): string|array {
    $definition = $plugin->getPluginDefinition();

    $label = $definition['label'] ?? NULL;
    $description = $definition['description'] ?? NULL;
    $url = $this->pluginDocUrl($plugin);

    if (!$url && !$description) {
      return $label;
    }

    $build = [];

    if ($url) {
      $build['title'] = [
        '#type' => 'link',
        '#title' => $label,
        '#url' => $url,
      ];
      $build['space'] = ['#markup' => ' '];
      $build['icon'] = [
        '#type' => 'link',
        '#title' => $this->icon,
        '#attributes' => ['title' => $label],
        '#url' => $url,
      ];
    }
    else {
      $build['title'] = [
        '#markup' => $label,
      ];
    }

    if ($description) {
      $build['description'] = [
        '#markup' => $description,
        '#prefix' => '<div>',
        '#suffix' => '</div>',
      ];
    }

    return ['data' => $build];
  }

  /**
   * Builds the default configuration for an ECA plugin.
   *
   * @param \Drupal\Core\Plugin\PluginBase|EcaPluginBase $plugin
   *   The ECA plugin instance.
   *
   * @return array
   *   A rendered array containing the plugin's configuration data.
   */
  protected function buildConfiguration(PluginBase|EcaPluginBase $plugin): array {
    if (!method_exists($plugin, 'getConfiguration')) {
      return [];
    }

    $configuration = $plugin->getConfiguration();
    return $configuration ? [
      'data' => [
        '#plain_text' => Yaml::encode($configuration),
        '#prefix' => '<small><pre>',
        '#suffixx' => '</pre></small>',
      ],
    ] : [];
  }

  /**
   * Append items to items by their provider (a.k.a., module name).
   *
   * @param array $items
   *   A reference to the array where items will be grouped and stored.
   * @param array $item
   *   The item to be added to the appropriate group.
   * @param array $definition
   *   The plugin definition, including the provider name (a.k.a., module name).
   *
   * @return void
   */
  protected function appendItem(array &$items, array $item, array $definition): void {
    $module_name = $definition['provider'];
    $items += [$module_name => []];
    $items[$module_name][] = $item;
  }

  /**
   * Builds an array of renderable items based on provided headers and rows.
   *
   * @param array $header
   *   An array defining headers for a table.
   * @param array $items
   *   An associative array of items where keys are module names and values are
   *   arrays of rows to be displayed in the table.
   *
   * @return array
   *   A structured array suitable for rendering, including module names and
   *   their associated tables.
   */
  protected function buildItems(array $header, array $items): array {
    $build = [];
    foreach ($items as $module_name => $rows) {
      $build[$module_name] = [];
      $build[$module_name]['name'] = [
        '#markup' => match ($module_name) {
          'core' => $this->t('Drupal core'),
          default => $this->moduleExtensionList->exists($module_name)
            ? $this->moduleExtensionList->getName($module_name)
            : $module_name
        },
        '#prefix' => '<h2>',
        '#suffix' => '</h2>',
      ];
      $build[$module_name]['table'] = [
        '#theme' => 'table',
        '#header' => $header,
        '#rows' => $rows,
      ];
    }
    return $build;
  }

  /**
   * Builds the URL to the offsite documentation for the given plugin.
   *
   * @param \Drupal\Component\Plugin\PluginInspectionInterface $plugin
   *   The plugin for which the documentation URL should be build.
   *
   * @return URL|null
   *   The URL to the offsite documentation, or NULL if no URL was generated.
   *
   * @see \Drupal\eca_modeller_bpmn\ModellerBpmnBase::pluginDocUrl
   */
  protected function pluginDocUrl(PluginInspectionInterface $plugin): ?Url {
    $domain = $this->documentationDomain;
    if (!$domain) {
      return NULL;
    }

    $provider = $plugin->getPluginDefinition()['provider'];
    $basePath = (mb_strpos($provider, 'eca_') === 0) ?
      str_replace('_', '/', $provider) :
      $provider;

    $uri = sprintf('%s/plugins/%s/%ss/%s/', $domain, $basePath, $this->pluginType, str_replace([':'], '_', $plugin->getPluginId()));

    return Url::fromUri($uri);
  }

}
